﻿/*--------------------------------------------------------------------------*/
/* deviceconfigurationleft.aspx.cs										    */
/*																			*/
/*																			*/
/*																			*/
/*																			*/
/*																			*/
/*																			*/
/*--------------------------------------------------------------------------*/

using System;
using System.Text;
using System.Drawing;
using System.Collections.Specialized;
using System.Collections.Generic;

// actimage
using actimage.framework.system;

// Application
using EndressHauser.InfoSys.Application;
using EndressHauser.InfoSys.Application.Session;
using EndressHauser.InfoSys.Application.Utils;
using EndressHauser.InfoSys.Application.Alarms;
using EndressHauser.InfoSys.Application.Variants;
using EndressHauser.InfoSys.Application.TreeView;

// Infragistics
using Infragistics.Web.UI.NavigationControls;


namespace EndressHauser.InfoSys.Website.Developer
{
    /// <summary>
    /// This page contains the device configuration treeview
    /// </summary>
    public partial class deviceconfigurationleft : MarketingWebPage
    {
        #region Fields
        protected IPDeviceConfigurationTree dc;
        private List<string> _UserDevices;
        #endregion

        #region Message Handlers
        /// <summary>
        /// Occurs when the page is loaded
        /// </summary>
        /// <param name="sender">The sender of the message</param>
        /// <param name="e">The arguments of the event</param>
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// Occurs when a node is clicked
        /// </summary>
        /// <param name="sender">The sender of the message</param>
        /// <param name="e">The arguments of the event</param>
        protected void wdtDeviceConfiguration_NodeClick(object sender, DataTreeNodeClickEventArgs e)
        {
            try
            {
                this.SelectNode(e.Node);
            }
            catch (InfoSysException ex)
            {
                SessionManager.TreeErrorMessage = ex.Message;

                if (e.Node.ParentNode != null)
                {
                    SessionManager.SelectedDeviceNodeTag = (string)e.Node.ParentNode.Key;
                }

                SessionManager.RefreshDeviceTree = true;

                Response.Redirect("deviceconfigurationleft.aspx");
            }
            finally
            {
                SaveTreeInSession();
            }
        }

        protected void wdtDeviceConfiguration_NodePopulate(object sender, DataTreeNodeEventArgs e)
        {
            DeviceConfigurationNode dcn;

            if (e.Node != null)
            {
                e.Node.Nodes.Clear();

                if (e.Node.DataPath.Length > 0)
                {
                    dcn = (DeviceConfigurationNode)dc.Tree.Root.Nodes[e.Node.DataPath];
                }
                else
                {
                    dcn = (DeviceConfigurationNode)dc.Tree.Root;
                }

                TreeNodeCollection nc = dc.Tree.GetChildNodes(e.Node.DataPath);

                if (nc != null)
                {
                    foreach (ITreeNode n in nc)
                    {
                        DeviceConfigurationNode dn = n as DeviceConfigurationNode;

                        if (n.Visible && !n.Type.Equals("Text"))
                        {
                            BuildNode(dn, e.Node.Nodes);
                        }
                    }

                    // when no nodes need to be added, we add a dummy node
                    //if ((nc.Count == 0) && (e.Node.ShowExpand))
                    //{
                    //    Node node = e.Node.Nodes.Add("Empty", "NoNodes");

                    //    node.DataPath = "Empty";
                    //    node.Hidden = false;
                    //    node.ShowExpand = false;

                    //    node.ImageUrl = "../Resources/folderIcon.png";
                    //}

                    //e.Node.Nodes.Sort(false, true);

                    if (dcn != null)
                    {
                        BuildMenuItems(dcn);

                        //this.uwtDeviceConfiguration.SelectedNode = e.Node;

                        //e.Node.Expand(true);


                    }
                }
                SaveTreeInSession();
            }
        }

        /// <summary>
        /// Occurs when a menu item is clicked
        /// </summary>
        /// <param name="sender">The sender of the message</param>
        /// <param name="e">The arguments of the event</param>
        protected void wdmContextMenu_ItemClick(object sender, DataMenuItemEventArgs e)
        {
            List<DataTreeNode> selectedNodes = this.wdtDeviceConfiguration.SelectedNodes;
            // to ensure that a node is selected when performing edit actions
            if (selectedNodes != null && selectedNodes.Count != 0)
            {
                try
                {
                    switch ((string)e.Item.Key)
                    {
                        case "Cut": // cut the selected node and all child nodes
                            dc.CutNode((string)selectedNodes[0].DataItem);

                            SessionManager.RefreshDeviceTree = true;

                            // save the tag of the selected node...
                            SessionManager.SelectedDeviceNodeTag = (string)selectedNodes[0].Key;

                            break;
                        case "Copy": // copy the selected node and all child nodes
                            dc.CopyNode((string)selectedNodes[0].DataItem);

                            SessionManager.RefreshDeviceTree = true;

                            // save the tag of the selected node...
                            SessionManager.SelectedDeviceNodeTag = (string)selectedNodes[0].Key;
                            break;
                        case "Paste": // paste a node and all child nodes
                            dc.PasteNode((string)selectedNodes[0].DataItem);
                            SessionState.setObject("pastedNode", selectedNodes[0]);

                            SessionManager.RefreshDeviceTree = true;

                            // save the tag of the selected node...
                            SessionManager.SelectedDeviceNodeTag = (string)selectedNodes[0].Key;

                            break;
                        case "Delete": // delete the selected node and all child nodes
                            dc.RemoveNode((string)selectedNodes[0].DataItem);

                            this.wdtDeviceConfiguration.SelectedNodes[0].ParentNode.Nodes.Clear();
                            this.wdtDeviceConfiguration_NodePopulate(this.wdtDeviceConfiguration,
                                new DataTreeNodeEventArgs(this.wdtDeviceConfiguration.SelectedNodes[0].ParentNode));

                            SessionManager.RefreshDeviceTree = true;

                            // save the tag of the selected node...
                            if (selectedNodes[0].ParentNode != null)
                            {
                                SessionManager.SelectedDeviceNodeTag = (string)selectedNodes[0].ParentNode.Key;
                            }
                            break;
                        case "Refresh": // refresh the selected node and all child nodes
                            this.wdtDeviceConfiguration_NodePopulate(this.wdtDeviceConfiguration,
                                new DataTreeNodeEventArgs(this.wdtDeviceConfiguration.SelectedNodes[0].ParentNode));

                            this.wdtDeviceConfiguration.SelectedNodes[0].ExpandChildren();

                            SessionManager.RefreshDeviceTree = true;

                            // save the tag of the selected node...
                            SessionManager.SelectedDeviceNodeTag = (string)selectedNodes[0].Key;
                            break;
                        default: // do nothing
                            break;
                    }
                }
                catch (InfoSysException iex)
                {
                    Exception ex = iex;

                    while (iex.Message.Length == 0)
                    {
                        ex = ex.InnerException;
                    }

                    SessionManager.TreeErrorMessage = ex.Message;
                }

                SaveTreeInSession();

                //  Response.Redirect("deviceconfigurationleft.aspx");
            }
        }
        #endregion

        #region Web Form Designer generated code
        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"></see> event to initialize the page.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"></see> that contains the event data.</param>
        override protected void OnInit(EventArgs e)
        {
            InitializeComponent();

            base.OnInit(e);

            wdtDeviceConfiguration.Nodes[0].IsEmptyParent = true;

            InitPage(EndressHauser.InfoSys.Application.Help.HelpTopic.None, ControllerName.DeviceConfiguration);

            dc = ModelProxy.GetInterface("IPDeviceConfigurationTree") as IPDeviceConfigurationTree;

            if (!this.IsPostBack)
            {
                Initialize(false, false, 0);

                GenerateTreeErrorMessage();

                NameValueCollection parameters = this.Page.Request.Params;
                bool wasNodeCollapsed = parameters.Keys[0].Equals("nodeCollapsed");
                bool needMenuBuild = parameters.Keys[0].Equals("buildMenu");

                // see if there are some request parameters
                if (parameters.Count > 0 && wasNodeCollapsed)
                {
                    // here we save the tree state when a node is collapsed

                    LoadTreeFromSession();


                    DataTreeNode n =
                        (DataTreeNode)this.wdtDeviceConfiguration.Nodes.FindNavItemByKey(parameters.Get(0), true);
                    if (n != null)
                    {
                        n.Expanded = false;
                    }

                    SaveTreeInSession();
                }
                else if (parameters.Count > 0 && needMenuBuild)
                {
                    DataTreeNode n =
                       (DataTreeNode)this.wdtDeviceConfiguration.Nodes.FindNavItemByKey(parameters.Get(0), true);
                    if (n != null)
                    {
                        DeviceConfigurationNode dcn = null;

                        if (n.DataPath.Length > 0)
                        {
                            dcn = (DeviceConfigurationNode)dc.Tree.Root.Nodes[n.DataPath];
                        }
                        else
                        {
                            dcn = (DeviceConfigurationNode)dc.Tree.Root;
                        }

                        BuildMenuItems(dcn);
                    }
                }
                else
                {
                    LoadData();
                }
            }


        }

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.ID = "deviceconfigurationleft";
        }
        #endregion

        #region Misc
        /// <summary>
        /// Load data
        /// </summary>
        private void LoadData()
        {
            SessionUser su = SessionManager.User;

            LoadTreeFromSession();

            if (SessionManager.UserDevices == null)
            {
                // retrieve all devices that are available for current user
                _UserDevices = dc.GetUserDevices(su.UserName);
                SessionManager.UserDevices = _UserDevices;
            }
            else
            {
                _UserDevices = SessionManager.UserDevices;
            }

            // if the tree is loaded for the first time, build the 
            if (SessionManager.FirstDeviceTreeLoad)
            {
                this.wdtDeviceConfiguration.Nodes[0].Nodes.Clear();
                this.wdtDeviceConfiguration_NodePopulate(this.wdtDeviceConfiguration,
                                           new DataTreeNodeEventArgs(this.wdtDeviceConfiguration.Nodes[0]));
                for (int i = 0; i < this.wdtDeviceConfiguration.Nodes[0].Nodes.Count; i++)
                {
                    if (!this.wdtDeviceConfiguration.Nodes[0].Nodes[i].Text.Equals("Empty"))
                    {
                        this.wdtDeviceConfiguration_NodePopulate(this.wdtDeviceConfiguration,
                                           new DataTreeNodeEventArgs(this.wdtDeviceConfiguration.Nodes[0].Nodes[i]));
                    }
                }

                this.SelectNode(this.wdtDeviceConfiguration.Nodes[0]);

                SessionManager.FirstDeviceTreeLoad = false;
            }

            if ((SessionManager.SelectedDeviceNodeTag != null) && (SessionManager.SelectedDeviceNodeTag.Length > 0) && (SessionManager.RefreshDeviceTree))
            {
                DataTreeNode pn = null;

                if (SessionManager.RefreshParentNodeLevel != 0)
                {
                    pn =
                        (DataTreeNode)this.wdtDeviceConfiguration.Nodes.FindNavItemByKey
                                            (SessionManager.SelectedDeviceNodeParentTag, true);
                    if (pn != null)
                    {
                        int i = 1;

                        DataTreeNode n = pn;

                        while (i < SessionManager.RefreshParentNodeLevel)
                        {
                            if (n != null)
                            {
                                n = n.ParentNode;
                            }
                            i++;
                        }

                        // reload nodes
                        this.wdtDeviceConfiguration_NodePopulate(n,
                           new DataTreeNodeEventArgs(n));
                        n.ExpandChildren();

                    }

                    DataTreeNode bn =
                        (DataTreeNode)this.wdtDeviceConfiguration.Nodes.FindNavItemByKey
                                                            (SessionManager.SelectedMenusNodeTag, true);
                    if (bn != null)
                    {
                        if (SessionManager.ErrorMessage.Length == 0)
                        {
                            this.SelectNode(bn);

                            //   this.uwtDeviceConfiguration.ScrollNodeIntoView(bn);
                        }
                    }

                    SessionManager.RefreshParentNodeLevel = 0;
                }
                else
                {
                    DataTreeNode n =
                         (DataTreeNode)this.wdtDeviceConfiguration.Nodes.FindNavItemByKey(SessionManager.SelectedMenusNodeTag, true);
                    if (n != null)
                    {
                        // reload nodes
                        this.wdtDeviceConfiguration_NodePopulate(n,
                           new DataTreeNodeEventArgs(n));
                        n.ExpandChildren();

                        if (SessionManager.ErrorMessage.Length == 0)
                        {
                            this.SelectNode(n);

                            // this.uwtDeviceConfiguration.ScrollNodeIntoView(n);
                        }
                    }
                    else
                    {
                        pn =
                            (DataTreeNode)this.wdtDeviceConfiguration.Nodes.FindNavItemByKey
                                                        (SessionManager.SelectedMenusNodeTag, true);
                        if (pn != null)
                        {
                            // reload nodes
                            this.wdtDeviceConfiguration_NodePopulate(pn,
                           new DataTreeNodeEventArgs(pn));
                            pn.ExpandChildren();

                            if (SessionManager.ErrorMessage.Length == 0)
                            {
                                this.SelectNode(n);

                                // this.uwtDeviceConfiguration.ScrollNodeIntoView(pn);
                            }
                        }
                    }
                }

                SessionManager.RefreshDeviceTree = false;
            }

            SaveTreeInSession();
        }

        /// <summary>
        /// Process a node selection in the tree
        /// </summary>
        /// <param name="node">The selected node</param>
        private void SelectNode(DataTreeNode node)
        {
            if ((node != null) && !node.Text.Equals("Empty"))
            {
                if (SessionManager.RefreshDeviceTree)
                {
                    this.wdtDeviceConfiguration_NodePopulate(node, new DataTreeNodeEventArgs(node));
                }

                string dataPath;

                if (node.Level > 0)
                {
                    dataPath = node.DataPath;
                }
                else
                {
                    dataPath = null;
                }

                DeviceConfigurationNode n = dc.SelectNode(dataPath);

                // build the context menu items
                BuildMenuItems(n);

                // save the tag of the selected node...
                SessionManager.SelectedDeviceNodeTag = (string)node.Key;

                // ... and also the tag of the parent node, in case the selected node is deleted
                if (node.ParentNode != null)
                {
                    SessionManager.SelectedDeviceNodeParentTag = (string)node.ParentNode.Key;
                }

                this.RefreshGUI();

                //this.uwtDeviceConfiguration.SelectedNode = node;

                BuildJavascript();

                SaveTreeInSession();
            }
        }

        /// <summary>
        /// Build a tree node
        /// </summary>
        /// <param name="n">The tree node</param>
        /// <param name="treeNodes">The parent nodes collection</param>
        private void BuildNode(
            DeviceConfigurationNode n,
            DataTreeNodeCollection treeNodes)
        {
            string nodeLabel = n.Label;
            string nodeIdentifier = n.Identifier;

            DataTreeNode node = treeNodes.Add(nodeLabel, nodeIdentifier);

            node.DataPath = nodeIdentifier;

            if (n.Tooltip.Count == 1)
            {
                node.ToolTip = n.Tooltip[0];
            }
            else
            {
                for (int i = 1; i < n.Tooltip.Count; i++)
                {
                    node.ToolTip += n.Tooltip[i];
                    if (i < n.Tooltip.Count - 1)
                    {
                        node.ToolTip += "\n";
                    }
                }
            }

            node.DataItem = nodeIdentifier;
            node.Visible = true;
            node.IsEmptyParent = n.HasChildren;

            node.ImageUrl = "../Resources/" + ImageMapping.ReadFromXml(n.Type);

            #region Special handling
            //DeviceAlarmNode dan = n as DeviceAlarmNode;
            //if (dan != null)
            //{
            //    if (nodeLabel.IndexOf("unused") != -1)
            //    {
            //        node.Style.ForeColor = Color.Red;
            //    }
            //}

            VariantNode vn = n as VariantNode;
            if (vn != null)
            {
                //if (vn.Frozen)
                //{
                //    node.Style.ForeColor = Color.CornflowerBlue;
                //}
                if (vn.Obsolete)
                {
                    node.Visible = false;
                }
            }

            //if (n.Type.Equals("InconsistentDependency"))
            //{
            //    node.Style.ForeColor = Color.Red;
            //}

            //if (n.MissingData && !n.Type.Equals("FunctionGroup"))
            //{
            //    node.Style.ForeColor = Color.Red;
            //}

            if (n.Type.Equals("Device"))
            {
                string deviceID = StringManager.Split(n.Identifier);
                // check if current device belongs to the user devices
                if ((deviceID != null) && !_UserDevices.Contains(deviceID))
                {
                    treeNodes.Remove((DataTreeNode)node);
                }
            }
            #endregion
        }

        /// <summary>
        /// Build all the menu items
        /// </summary>
        /// <param name="node">The node</param>
        private void BuildMenuItems(
            DeviceConfigurationNode node)
        {
            if (node != null)
            {
                // retrieve allowed commands for current node
                Array commands = node.Commands;

                // hide all menu items
                foreach (DataMenuItem i in this.wdmContextMenu.Items)
                {
                    i.Visible = false;
                }

                string d = SessionManager.DeviceId;
                // a device or one of its sub nodes is selected
                if (d != null)
                {
                    string deviceID = StringManager.Split(d);
                    // check if current device belongs to the user devices
                    if ((deviceID != null) && _UserDevices.Contains(deviceID))
                    {
                        if (node.AllowUpdate)
                        {
                            // if selected node has some commands, add them to the menu
                            if ((commands.Length > 0) && (this.wdtDeviceConfiguration.SelectedNodes != null) && (this.wdtDeviceConfiguration.SelectedNodes.Count != 0))
                            {
                                foreach (TreeCommand tc in commands)
                                {
                                    string menuItemName = tc.ToString();

                                    this.wdmContextMenu.Items.FindNavItemByKey(menuItemName).Visible = true;

                                    if (menuItemName.Equals("Paste") && !dc.CommandManager.CanPaste)
                                    {
                                        this.wdmContextMenu.Items.FindNavItemByKey(menuItemName).Visible = false;
                                    }
                                }
                            }
                        }
                    }
                }
                else // the 'Root' node is selected
                {
                    if (commands.Length > 0)
                    {
                        foreach (TreeCommand tc in commands)
                        {
                            string s = tc.ToString();
                            string firstCharacter = s.Substring(0, 1);
                            string rest = s.Substring(1, s.Length - 1);
                            rest = rest.ToLower();
                            string menuItemName = firstCharacter + rest;
                            this.wdmContextMenu.Items.FindNavItemByKey(menuItemName).Visible = false;
                        }
                    }
                }

                // show standard commands
                this.wdmContextMenu.Items.FindNavItemByKey("Expand_Collapse").Visible = true;
                this.wdmContextMenu.Items.FindNavItemByText("Refresh").Visible = true;

                this.wdmContextMenu.Items[0].Text = node.Label;
                this.wdmContextMenu.Items[0].Visible = true;

                // bind data
                this.wdmContextMenu.DataBind();
            }
        }

        /// <summary>
        /// Save device configuration tree in the session, so that its state (nodes expanded/collapses) is saved
        /// </summary>
        private void LoadTreeFromSession()
        {
            // load the tree from session if it already exists
            if (SessionState.getObject("dctree") != null)
            {
                WebDataTree t = (WebDataTree)SessionState.getObject("dctree");
                this.wdtDeviceConfiguration.Nodes.Clear();
                foreach (DataTreeNode n in t.Nodes)
                    this.wdtDeviceConfiguration.Nodes.Add(n);
            }
        }

        /// <summary>
        /// Save device configuration tree in the session, so that its state (nodes expanded/collapses) is saved
        /// </summary>
        private void SaveTreeInSession()
        {
            SessionState.setObject("dctree", this.wdtDeviceConfiguration);
        }

        /// <summary>
        /// Builds the javascript.
        /// </summary>
        private void BuildJavascript()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("<script language=Javascript>");
            sb.Append("parent.frames(0).location = 'navigation.aspx?tree=dctree&treetype=deviceconfiguration'");
            sb.Append(";");
            sb.Append("</script>");

            if (!ClientScript.IsClientScriptBlockRegistered("clientScript"))
            {
                Page.ClientScript.RegisterClientScriptBlock(typeof(string), "clientScript", sb.ToString());
            }
        }
        #endregion






    }
}
